% January 2015, F. DE FIORE and H. UHLIG. "Corporate debt structure and the financial crisis", JMCB 2015 and ECB WP No. 1759.
% Choice.m calculates the choice of the entrepreneur of whether
% to go to the bank or to finance him/herself via CMF or dropping out
% Choice.m also calculates the fraction of entrepreneurs in either
% of these three categories
% The important aggregate variable is s_aggr, the overall productivity
% level. NB: there is a change of notation. The variable q in the paper corresponds
% to s_aggr in these codes.
%
% Random shocks are
% - eps1, which is drawn before making choice
% - eps2, which is drawn after choice of CMF vs bank vs abstain,
%       but observed by entrepreneurs at banks.
%       Object of CSV for CMFs
% - eps3, uncertain, and only known after final production.
%       Object of CSV for banks and CMFs


DO_CHECK_BOUNDS = 0;
DO_PLOT_IN_CHOICE = 0;
if ~exist('DOING_SEARCH'),
    DOING_SEARCH = 0;
end;

s = 1;
if ~exist('s_aggr'),
    s_aggr = 1.8; 
end;

s_bot = exp(- sigma_eps1^2/2 - 10*sigma_eps1); 
s_top = exp(- sigma_eps1^2/2 + 10*sigma_eps1); 
s_very_high = 50*s_aggr;


% First: calculate profits from going to CMFs
% Residual uncertainty: coming from eps3 and eps2

sigma_cmf = sqrt( sigma_eps3^2 + sigma_eps2^2);
sigma = sigma_cmf;
solve_s;
sgrd_cmf = sgrd;
om_cmf = omsolve;
profits_cmf = profits;
frac_cmf_monitored = frac_mon;


% Second: calculating profits from going to bank.
% Once an entrepreneur has arrived at the bank,
% the residual uncertainty is coming from eps3

bank;
profits_bank = (1-tau)*profbank;
sgrd_bank = s_times_eps1_grd;


DO_2STD_DEV_LINES = 0;
if ~DOING_ESTIMATION,
    if DO_PLOT_IN_CHOICE
        plot(sgrd_cmf,profits_cmf,'g-',sgrd_bank,profits_bank,'r-',[sgrd_bank,sgrd_cmf],1+0*[sgrd_bank,sgrd_cmf],'k-');
        axis([.99 1.034 0.94 1.06])
        if DO_2STD_DEV_LINES,
            hold on
            plot([s_aggr-2*sigma_eps1,s_aggr-2*sigma_eps1],[0.5,2.5],[s_aggr+2*sigma_eps1,s_aggr+2*sigma_eps1],[0.5,2.5]);
            hold off
        end;
        title(sprintf('s-aggr = %12.8f',s_aggr));
    end;
end;

crit1 = min(profits_bank) <1;
crit2 = min(profits_cmf)<1;
crit3 = max(profits_bank) >1;
crit4 = max(profits_cmf) > 1;
cri = crit1*crit2*crit3*crit4;

    if (~DOING_ESTIMATION) & (~DOING_SEARCH),
        if crit1*crit2*crit3*crit4,
            disp('four criteria are satisfied!');
        end;
    end;
    
    s_ng_stp = (s_top-s_bot)/20000; 
    s_ng = s_bot : s_ng_stp : s_top;
    res = interp1(sgrd_bank,profits_bank,s_ng) > 1;
    res2 = interp1(sgrd_cmf,profits_cmf,s_ng) > interp1(sgrd_bank,profits_bank,s_ng);
    res3 = interp1(sgrd_cmf,profits_cmf,s_ng) > 1;
    
    [dummy,j_bank]=max(res);
    [dummy,j_cmf]=max(res2);
    if ~exist('DO_NO_CMF_IN_CHOICE'),
        DO_NO_CMF_IN_CHOICE = 0;
    end;
         
    if j_bank < j_cmf,
        s_bank = s_ng(j_bank);
        s_cmf = s_ng(j_cmf);
    else
        j_bank = j_cmf;
        s_bank = s_ng(j_bank);
        s_cmf = s_ng(j_cmf);
    end;
   if DO_NO_CMF_IN_CHOICE, % only the very largest epsilon-1 entrepreneur gets CMF,
        % thus, effectively shutting down CMF funding
        j_cmf = round(length(s_ng)*0.98); % i.e. treshold at four and a half std dev
        s_cmf = s_ng(j_cmf);
    end;

    res2 = max( res2, s_ng > s_cmf );
    
    undef_bank = isnan( interp1(sgrd_bank,profits_bank,s_ng) );
    undef_cmf  = isnan( interp1(sgrd_cmf,profits_cmf,s_ng) );
    
    if ~DOING_ESTIMATION,
        if DO_CHECK_BOUNDS,
            plot(s_ng,.92*undef_bank+.11,'k-',s_ng,.77*res.*(1-res2)+.21,'r--');
            title('bank range');
            disp('hit any key when ready...');pause;
            plot(s_ng,.92*undef_cmf+.11,'k-',s_ng,.77*res2.*res3+.21,'r--');
            title('cmf range');
            disp('hit any key when ready...');pause;
        end;
    end;
    
    prob_of_rectangle_original = (dens((s_ng/s_aggr+s_ng_stp/2),sigma_eps1)/s_aggr)*s_ng_stp;
    prob_of_rectangle = prob_of_rectangle_original/sum(prob_of_rectangle_original);
    
    %compute shares: abstain, bank finance, cmf finance
    share_abstain = sum((1-undef_cmf).*(1-res3).*(1-undef_bank).*(1-res).*prob_of_rectangle);
    share_abstain_check = quad(@dens,0.0001,s_bank/s_aggr,[],[],sigma_eps1);
    share_bank = sum((1-undef_bank).*res.*(1-res2).*prob_of_rectangle);
    share_bank_check = quad(@dens,s_bank/s_aggr,s_cmf/s_aggr,[],[],sigma_eps1);
    share_cmf =  sum((1-undef_cmf).*res2.*res3.*prob_of_rectangle);
    share_cmf_check = quad(@dens,s_cmf/s_aggr,s_very_high/s_aggr,[],[],sigma_eps1);
    
    %Compute shares conditional on the financing choice: dropouts and monitoring for bank finance, monitoring for CMF finance
    %frac_bank_dropout is computed in bank.m as normcdf( log( eps2_cutoff + sigma_eps2^2/2)/sigma_eps2 );
    frac_bank_dropout_withchoice = ...
        (1-undef_bank).*res.*(1-res2).*interp1([s_bot-1,sgrd_bank,s_top+1],[0,frac_bank_dropout,0],s_ng);
    frac_bank_continue_withchoice = ...
        (1-undef_bank).*res.*(1-res2).*interp1([s_bot-1,sgrd_bank,s_top+1],[0,frac_bank_continue,0],s_ng);
    frac_bank_monitored_withchoice = ...
        (1-undef_bank).*res.*(1-res2).*interp1([s_bot-1,sgrd_bank,s_top+1],[0,frac_bank_monitored,0],s_ng);
    frac_cmf_monitored_withchoice = ...
        (1-undef_cmf).*res2.*res3.*interp1([s_bot-1,sgrd_cmf,s_top+1],[0,frac_cmf_monitored,0],s_ng);
    
    %compute unconditional shares: dropouts, monitoring, repayments
    expected_bank_dropouts = sum((1-undef_bank).*res.*(1-res2).*frac_bank_dropout_withchoice.*prob_of_rectangle);
    expected_bank_monitoring = sum(frac_bank_monitored_withchoice.*prob_of_rectangle);
    expected_bank_repay = share_bank-expected_bank_dropouts-expected_bank_monitoring;
    expected_cmf_monitoring = sum(frac_cmf_monitored_withchoice.*prob_of_rectangle);
    expected_cmf_repay = share_cmf-expected_cmf_monitoring;
    
    cond_dropout_if_banking = expected_bank_dropouts/(share_bank+.00001);
    cond_monitored_if_banking = expected_bank_monitoring/(share_bank+.00001);
    cond_monitored_if_continue_and_banking = expected_bank_monitoring/(share_bank*(1-cond_dropout_if_banking)+.00001);
    cond_monitored_if_cmf     = expected_cmf_monitoring/(share_cmf+.00001);
    
    %compute expected profits conditional on the realisation of eps1
    overall_profits = max( [1+0*s_ng,
        interp1(sgrd_bank,profits_bank,s_ng),
        interp1(sgrd_cmf,profits_cmf,s_ng)]);
    expected_profits = sum(overall_profits.*prob_of_rectangle);

